@@ -344,6 +344,79 @@ async fn provider(mut req: Request<()>) -> tide::Result {
     }
 }
 
+async fn get_url_object(req: Request<()>) -> tide::Result {
+    let token_value = req.header("authorization");
+    match token_value {
+        Some(token_header) => {
+            let token = token_header.last().to_string();
+            let (check, username) = jwt_verify(token, "get_object".to_string()).await?;
+            if check {
+
+                let region = req.param("region")?.to_string();
+                let bucket = req.param("bucket")?.to_string();
+                let object_path = req.param("object_path")?.to_string();
+
+                let mut provider_maybe_name: Option<String> = None;
+                for rg in PROVIDERS.iter() {
+                    if rg.val().contains(&region) {
+                        provider_maybe_name = Some(rg.key().to_string());
+                    }
+                }
+
+                match provider_maybe_name {
+                    Some(provider_name) => {
+                        let provider_value = get_provider_by_name(provider_name, username)?;
+
+                        match provider_value {
+                            Some(provider) => {
+                                let bucket_name = bucket;
+                                let region = region.parse()?;
+                            
+                                let access_key = provider.access_key;
+                                let secret_key = provider.secret_key;
+                                let credentials = Credentials::new(Some(&access_key), Some(&secret_key), None, None, None)?;
+                            
+                                let bucket = Bucket::new(&bucket_name, region, credentials)?;
+
+                                let (data, code) = bucket.get_object(object_path.clone()).await?;
+
+                                let content_type;
+                                match infer::get(&data) {
+                                    Some(kind) => {
+                                        content_type = kind.mime_type();
+                                    },
+                                    None => {
+                                        content_type = "application/octet-stream";
+                                    }
+                                }
+
+                                return Ok(tide::Response::builder(code).body(data)
+                                    .header("content-type", content_type)
+                                    .build());
+                            },
+                            None => {
+                                let j = json!({"error": "region not valid for provider"});
+                                println!("{}", j);
+                                return Ok(tide::Response::builder(400).body(j).header("content-type", "application/json").build());
+                            }
+                        }
+                    },
+                    None => {
+                        let j = json!({"error": "region not valid for provider"});
+                        println!("{}", j);
+                        return Ok(tide::Response::builder(400).body(j).header("content-type", "application/json").build());
+                    }
+                }
+            } else {
+                Ok(tide::Response::builder(401).header("content-type", "application/json").build())
+            }
+        },
+        None => {
+            Ok(tide::Response::builder(401).header("content-type", "application/json").build())
+        }
+    }
+}
+
 async fn get_object(mut req: Request<()>) -> tide::Result {
     let token_value = req.header("authorization");
     match token_value {
@@ -363,7 +436,7 @@ async fn get_object(mut req: Request<()>) -> tide::Result {
 
                 match provider_maybe_name {
                     Some(provider_name) => {
-                        let provider_value  = get_provider_by_name(provider_name, username)?;
+                        let provider_value = get_provider_by_name(provider_name, username)?;
 
                         match provider_value {
                             Some(provider) => {
@@ -375,9 +448,22 @@ async fn get_object(mut req: Request<()>) -> tide::Result {
                                 let credentials = Credentials::new(Some(&access_key), Some(&secret_key), None, None, None)?;
                             
                                 let bucket = Bucket::new(&bucket_name, region, credentials)?;
-                                let (data, code) = bucket.get_object(object_form.object_path).await?;
 
-                                return Ok(tide::Response::builder(code).body(data).header("content-type", "application/octet-stream").build());
+                                let (data, code) = bucket.get_object(object_form.object_path.clone()).await?;
+
+                                let content_type;
+                                match infer::get(&data) {
+                                    Some(kind) => {
+                                        content_type = kind.mime_type();
+                                    },
+                                    None => {
+                                        content_type = "application/octet-stream";
+                                    }
+                                }
+
+                                return Ok(tide::Response::builder(code).body(data)
+                                    .header("content-type", content_type)
+                                    .build());
                             },
                             None => {
                                 let j = json!({"error": "region not valid for provider"});
@@ -744,6 +830,7 @@ async fn main() -> tide::Result<()> {
     app.at("/create_bucket").post(create_bucket);
     app.at("/delete_bucket").post(delete_bucket);
     app.at("/list_bucket").post(list_bucket);
+    app.at("/files/:region/:bucket/:object_path").get(get_url_object);
 
     let ip = format!("0.0.0.0:{}", configure.port);
 
